@@ -38,19 +38,19 @@ namespace Banshee.IO.Unix
 {
     public class Directory : IDirectory
     {
-        [System.Runtime.InteropServices.DllImport ("libglib-2.0.dll")]
+        [System.Runtime.InteropServices.DllImport ("libglib-2.0-0.dll")]
         private static extern int g_mkdir_with_parents (IntPtr path, int mode);
-    
+
         public void Create (string directory)
         {
             IntPtr path_ptr = IntPtr.Zero;
-            
+
             try {
                 path_ptr = GLib.Marshaller.StringToPtrGStrdup (directory);
                 if (path_ptr == IntPtr.Zero) {
                     throw new Exception ("Failed to allocate native directory string");
                 }
-            
+
                 // 493 == 0755 - C# doesn't do octal literals
                 if (g_mkdir_with_parents (path_ptr, 493) == -1) {
                     Mono.Unix.UnixMarshal.ThrowExceptionForLastError ();
@@ -64,18 +64,18 @@ namespace Banshee.IO.Unix
                 }
             }
         }
-        
+
         public void Delete (string directory)
         {
             Delete (directory, false);
         }
-        
+
         public void Delete (string directory, bool recursive)
         {
             UnixDirectoryInfo unix_dir = new UnixDirectoryInfo (directory);
             unix_dir.Delete (recursive);
         }
-        
+
         public bool Exists (string directory)
         {
             try {
@@ -85,27 +85,55 @@ namespace Banshee.IO.Unix
                 return false;
             }
         }
-        
+
         public IEnumerable<string> GetFiles (string directory)
         {
-            UnixDirectoryInfo unix_dir = new UnixDirectoryInfo (directory);
-            foreach (UnixFileSystemInfo entry in unix_dir.GetFileSystemEntries ()) {
-                if (!entry.IsDirectory && entry.IsRegularFile && !entry.IsSocket && entry.Exists) {
+            var unix_dir = TraverseSymlink(new UnixDirectoryInfo (directory)) as UnixDirectoryInfo;
+            if (unix_dir == null) {
+                yield break;
+            }
+            foreach (var entry in unix_dir.GetFileSystemEntries ()) {
+                if (entry != null && !entry.IsDirectory && entry.IsRegularFile && !entry.IsSocket && entry.Exists) {
                     yield return entry.FullName;
                 }
             }
         }
-        
+
         public IEnumerable<string> GetDirectories (string directory)
         {
-            UnixDirectoryInfo unix_dir = new UnixDirectoryInfo (directory);
-            foreach (UnixFileSystemInfo entry in unix_dir.GetFileSystemEntries ()) {
-                if (entry.IsDirectory && entry.Exists && !entry.IsSocket) {
-                    yield return entry.FullName;
+            var unix_dir = new UnixDirectoryInfo (directory);
+            foreach (var entry in unix_dir.GetFileSystemEntries ()) {
+                var info = TraverseSymlink (entry);
+                if (info != null && info.IsDirectory && info.Exists && !info.IsSocket) {
+                    yield return info.FullName;
                 }
             }
         }
-        
+
+        private readonly HashSet<string> visited_symlinks = new HashSet<string> ();
+        private UnixFileSystemInfo TraverseSymlink (UnixFileSystemInfo info)
+        {
+            lock (visited_symlinks) {
+                visited_symlinks.Clear ();
+                while (info.IsSymbolicLink) {
+                    if (visited_symlinks.Contains (info.FullName)) {
+                        return null;
+                    }
+                    visited_symlinks.Add (info.FullName);
+                    var target = new UnixSymbolicLinkInfo (info.FullName).GetContents ();
+                    if (info.FullName.StartsWith (target.FullName)) {
+                        return null;
+                    }
+                    if (!target.Exists) {
+                        return null;
+                    }
+                    info = target;
+                }
+
+                return info;
+            }
+        }
+
         public void Move (SafeUri from, SafeUri to)
         {
             Mono.Unix.Native.Stdlib.rename (from.LocalPath, to.LocalPath);
