@@ -42,6 +42,7 @@ using Banshee.Metadata;
 using Banshee.Configuration;
 using Banshee.Collection;
 using Banshee.Equalizer;
+using Banshee.Collection.Database;
 
 namespace Banshee.MediaEngine
 {
@@ -230,6 +231,8 @@ namespace Banshee.MediaEngine
 
             if (args.Event == PlayerEvent.StartOfStream) {
                 incremented_last_played = false;
+            } else if (args.Event == PlayerEvent.EndOfStream) {
+                IncrementLastPlayed ();
             }
 
             RaiseEvent (args);
@@ -281,12 +284,35 @@ namespace Banshee.MediaEngine
 
         public void Open (SafeUri uri)
         {
-            OpenCheck (uri);
+             // Check if the uri exists
+            if (uri == null || !File.Exists (uri.AbsolutePath)) {
+                return;
+            }
+
+            bool found = false;
+            if (ServiceManager.DbConnection != null) {
+                // Try to find uri in the library
+                int track_id = DatabaseTrackInfo.GetTrackIdForUri (uri);
+                if (track_id > 0) {
+                    DatabaseTrackInfo track_db = DatabaseTrackInfo.Provider.FetchSingle (track_id);
+                    found = true;
+                    Open (track_db);
+                }
+            }
+            
+            if (!found) {
+                // Not in the library, get info from the file
+                TrackInfo track = new TrackInfo ();
+                using (var file = StreamTagger.ProcessUri (uri)) {
+                    StreamTagger.TrackInfoMerge (track, file, false);
+                }
+                Open (track);
+            }
         }
 
         void IPlayerEngineService.Open (string uri)
         {
-            OpenCheck (new SafeUri (uri));
+            Open (new SafeUri (uri));
         }
 
         public void SetNextTrack (TrackInfo track)
@@ -341,7 +367,7 @@ namespace Banshee.MediaEngine
             }
 
             try {
-                OpenCheck (track, true);
+                OpenCheck (track, play);
             } catch (Exception e) {
                 Log.Exception (e);
                 Log.Error (Catalog.GetString ("Problem with Player Engine"), e.Message, true);
@@ -350,11 +376,6 @@ namespace Banshee.MediaEngine
             }
         }
 
-        private void OpenCheck (object o)
-        {
-            OpenCheck (o, false);
-        }
-
         private void OpenCheck (object o, bool play)
         {
             if (CurrentState == PlayerState.NotReady) {
@@ -375,6 +396,11 @@ namespace Banshee.MediaEngine
                 return;
             }
 
+            if (track != null && (track.MediaAttributes & TrackMediaAttributes.ExternalResource) != 0) {
+                RaiseEvent (new PlayerEventArgs (PlayerEvent.EndOfStream));
+                return;
+            }
+
             PlayerEngine supportingEngine = FindSupportingEngine (uri);
             SwitchToEngine (supportingEngine);
             CheckPending ();
@@ -450,7 +476,6 @@ namespace Banshee.MediaEngine
         public void Close (bool fullShutdown)
         {
             IncrementLastPlayed ();
-            active_engine.Reset ();
             active_engine.Close (fullShutdown);
         }
 
@@ -535,6 +560,11 @@ namespace Banshee.MediaEngine
                 CurrentState == PlayerState.Contacting;
         }
 
+        public string GetSubtitleDescription (int index)
+        {
+            return active_engine.GetSubtitleDescription (index);
+        }
+
         private void CheckPending ()
         {
             if (pending_engine != null && pending_engine != active_engine) {
@@ -548,7 +578,9 @@ namespace Banshee.MediaEngine
         }
 
         public TrackInfo CurrentTrack {
-            get { return active_engine.CurrentTrack ?? synthesized_contacting_track; }
+            get {
+                return active_engine == null ? null : active_engine.CurrentTrack ?? synthesized_contacting_track;
+            }
         }
 
         private Dictionary<string, object> dbus_sucks;
@@ -625,6 +657,19 @@ namespace Banshee.MediaEngine
             get { return ((active_engine is IEqualizer) && active_engine.SupportsEqualizer); }
         }
 
+        public int SubtitleCount {
+            get { return active_engine.SubtitleCount; }
+        }
+
+        public int SubtitleIndex {
+            set { active_engine.SubtitleIndex = value; }
+        }
+
+        public SafeUri SubtitleUri {
+            set { active_engine.SubtitleUri = value; }
+            get { return active_engine.SubtitleUri; }
+        }
+
         public VideoDisplayContextType VideoDisplayContextType {
             get { return active_engine.VideoDisplayContextType; }
         }
@@ -691,7 +736,8 @@ namespace Banshee.MediaEngine
             | PlayerEvent.Volume
             | PlayerEvent.Metadata
             | PlayerEvent.TrackInfoUpdated
-            | PlayerEvent.RequestNextTrack;
+            | PlayerEvent.RequestNextTrack
+            | PlayerEvent.PrepareVideoWindow;
 
         private const PlayerEvent event_default_mask = event_all_mask & ~PlayerEvent.Iterate;
 
@@ -779,7 +825,11 @@ namespace Banshee.MediaEngine
                 LinkedListNode<PlayerEventHandlerSlot> node = event_handlers.First;
                 while (node != null) {
                     if ((node.Value.EventMask & args.Event) == args.Event) {
-                        node.Value.Handler (args);
+                        try {
+                            node.Value.Handler (args);
+                        } catch (Exception e) {
+                            Log.Exception (String.Format ("Error running PlayerEngine handler for {0}", args.Event), e);
+                        }
                     }
                     node = node.Next;
                 }
