@@ -25,7 +25,7 @@
 // OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
- 
+
 using System;
 using System.IO;
 using System.Text;
@@ -41,16 +41,16 @@ namespace Banshee.MediaProfiles
     {
         private bool profile_available = true;
         private Profile profile;
-        
+
         public TestProfileArgs(Profile profile)
         {
             this.profile = profile;
         }
-        
+
         public Profile Profile {
             get { return profile; }
         }
-        
+
         public bool ProfileAvailable {
             get { return profile_available; }
             set { profile_available = value; }
@@ -64,16 +64,31 @@ namespace Banshee.MediaProfiles
         internal static System.Globalization.CultureInfo CultureInfo {
             get { return System.Globalization.CultureInfo.InvariantCulture; }
         }
-    
+
         private XmlDocument document;
-        private List<Profile> profiles = new List<Profile>();
-        private Dictionary<string, PipelineVariable> preset_variables = new Dictionary<string, PipelineVariable>();
+        private List<Profile> profiles;
+        private Dictionary<string, PipelineVariable> preset_variables;
 
         public event TestProfileHandler TestProfile;
+        public event EventHandler Initialized;
 
         public MediaProfileManager()
         {
-            string path = Banshee.Base.Paths.GetInstalledDataDirectory ("audio-profiles");
+        }
+
+        private void Initialize ()
+        {
+            if (profiles != null) {
+                return;
+            }
+
+
+            uint timer = Hyena.Log.DebugTimerStart ();
+
+            profiles = new List<Profile> ();
+            preset_variables = new Dictionary<string, PipelineVariable> ();
+
+            string path = Hyena.Paths.GetInstalledDataDirectory ("audio-profiles");
             if(File.Exists(path)) {
                 LoadFromFile(path);
             } else if(Directory.Exists(path)) {
@@ -81,15 +96,22 @@ namespace Banshee.MediaProfiles
                 if(File.Exists(base_file)) {
                     LoadFromFile(base_file);
                 }
-                
+
                 foreach(string file in Directory.GetFiles(path, "*.xml")) {
                     if(Path.GetFileName(file) != "base.xml") {
                         LoadFromFile(file);
                     }
                 }
             }
+
+            Hyena.Log.DebugTimerPrint (timer, "Initialized MediaProfileManager: {0}");
+
+            EventHandler handler = Initialized;
+            if (handler != null) {
+                handler (this, EventArgs.Empty);
+            }
         }
-        
+
         private void LoadFromFile(string path)
         {
             document = new XmlDocument();
@@ -100,7 +122,7 @@ namespace Banshee.MediaProfiles
             } catch(Exception e) {
                 Console.WriteLine("Could not load profile: {0}\n{1}", path, e);
             }
-            
+
             document = null;
         }
 
@@ -115,7 +137,7 @@ namespace Banshee.MediaProfiles
             if(node == null) {
                 return;
             }
-            
+
             foreach(XmlNode variable_node in node.SelectNodes("variable")) {
                 try {
                     PipelineVariable variable = new PipelineVariable(variable_node);
@@ -145,6 +167,7 @@ namespace Banshee.MediaProfiles
         private Dictionary<string, string> mimetype_extensions = new Dictionary<string, string> ();
         public void Add(Profile profile)
         {
+            Initialize ();
             foreach (string mimetype in profile.MimeTypes) {
                 mimetype_extensions[mimetype] = profile.OutputFileExtension;
             }
@@ -153,6 +176,7 @@ namespace Banshee.MediaProfiles
 
         public void Remove(Profile profile)
         {
+            Initialize ();
             profiles.Remove(profile);
         }
 
@@ -162,16 +186,17 @@ namespace Banshee.MediaProfiles
                 throw new ArgumentNullException("id");
             }
 
+            Initialize ();
             return preset_variables[id];
         }
-        
+
         protected virtual bool OnTestProfile(Profile profile)
         {
             TestProfileHandler handler = TestProfile;
             if(handler == null) {
                 return true;
             }
-            
+
             TestProfileArgs args = new TestProfileArgs(profile);
             handler(this, args);
             return args.ProfileAvailable;
@@ -179,29 +204,42 @@ namespace Banshee.MediaProfiles
 
         public IEnumerable<Profile> GetAvailableProfiles()
         {
+            Initialize ();
+
             foreach(Profile profile in this) {
                 if(profile.Available == null) {
                     profile.Available = OnTestProfile(profile);
                 }
-                
+
                 if(profile.Available == true) {
                     yield return profile;
                 }
             }
         }
-        
+
         public ProfileConfiguration GetActiveProfileConfiguration (string id)
         {
+            Initialize ();
+
             return ProfileConfiguration.LoadActive (this, id);
         }
-        
+
         public ProfileConfiguration GetActiveProfileConfiguration(string id, string [] mimetypes)
         {
+            Initialize ();
+
             ProfileConfiguration config = GetActiveProfileConfiguration (id);
-            if(config != null) {
-                return config;
+            if (config != null) {
+                // Ensure the profile configuration is valid for the mimetype restriction
+                foreach (string profile_mimetype in config.Profile.MimeTypes) {
+                    foreach (string mimetype in mimetypes) {
+                        if (mimetype == profile_mimetype) {
+                            return config;
+                        }
+                    }
+                }
             }
-            
+
             foreach(string mimetype in mimetypes) {
                 Profile profile = GetProfileForMimeType(mimetype);
                 if(profile != null) {
@@ -209,25 +247,29 @@ namespace Banshee.MediaProfiles
                     return profile.Configuration;
                 }
             }
-            
+
             return null;
         }
-        
+
         public void TestAll()
         {
+            Initialize ();
+
             foreach(Profile profile in this) {
                 profile.Available = OnTestProfile(profile);
             }
         }
-        
+
         public Profile GetProfileForMimeType(string mimetype)
         {
+            Initialize ();
+
             foreach(Profile profile in GetAvailableProfiles()) {
                 if(profile.HasMimeType(mimetype)) {
                     return profile;
                 }
             }
-            
+
             return null;
         }
 
@@ -238,7 +280,9 @@ namespace Banshee.MediaProfiles
 
             if (extension[0] == '.')
                 extension = extension.Substring (1, extension.Length - 1);
-            
+
+            Initialize ();
+
             foreach (Profile profile in this) {
                 if (profile.OutputFileExtension == extension) {
                     return profile;
@@ -249,6 +293,8 @@ namespace Banshee.MediaProfiles
 
         public string GetExtensionForMimeType (string mimetype)
         {
+            Initialize ();
+
             if (mimetype != null && mimetype_extensions.ContainsKey (mimetype))
                 return mimetype_extensions[mimetype];
             return null;
@@ -256,20 +302,24 @@ namespace Banshee.MediaProfiles
 
         public IEnumerator<Profile> GetEnumerator()
         {
+            Initialize ();
             return profiles.GetEnumerator();
         }
 
         IEnumerator IEnumerable.GetEnumerator()
         {
+            Initialize ();
             return profiles.GetEnumerator();
         }
-        
+
         public int ProfileCount {
-            get { return profiles.Count; }
+            get { Initialize (); return profiles.Count; }
         }
-        
+
         public int AvailableProfileCount {
             get {
+                Initialize ();
+
                 int count = 0;
                 #pragma warning disable 0168, 0219
                 foreach(Profile profile in GetAvailableProfiles()) {
@@ -283,23 +333,24 @@ namespace Banshee.MediaProfiles
         string Banshee.ServiceStack.IService.ServiceName {
             get { return "MediaProfileManager"; }
         }
-        
+
         public override string ToString()
         {
+            Initialize ();
             StringBuilder builder = new StringBuilder();
-            
+
             builder.Append("Preset Pipeline Variables:\n\n");
             foreach(PipelineVariable variable in preset_variables.Values) {
                 builder.Append(variable);
                 builder.Append("\n");
             }
-            
+
             builder.Append("Profiles:\n\n");
             foreach(Profile profile in profiles) {
                 builder.Append(profile);
                 builder.Append("\n\n");
             }
-            
+
             return builder.ToString().Trim();
         }
     }
