@@ -61,7 +61,7 @@ namespace Banshee.PlaybackController
         private uint error_transition_id;
         private DateTime source_auto_set_at = DateTime.MinValue;
 
-        private PlaybackShuffleMode shuffle_mode;
+        private string shuffle_mode;
         private PlaybackRepeatMode repeat_mode;
         private bool stop_when_finished = false;
 
@@ -80,7 +80,7 @@ namespace Banshee.PlaybackController
         public event EventHandler NextSourceChanged;
         public event EventHandler TrackStarted;
         public event EventHandler Transition;
-        public event EventHandler<EventArgs<PlaybackShuffleMode>> ShuffleModeChanged;
+        public event EventHandler<EventArgs<string>> ShuffleModeChanged;
         public event EventHandler<EventArgs<PlaybackRepeatMode>> RepeatModeChanged;
 
         public PlaybackControllerService ()
@@ -90,6 +90,7 @@ namespace Banshee.PlaybackController
             player_engine = ServiceManager.PlayerEngine;
             player_engine.PlayWhenIdleRequest += OnPlayerEnginePlayWhenIdleRequest;
             player_engine.ConnectEvent (OnPlayerEvent,
+                PlayerEvent.RequestNextTrack |
                 PlayerEvent.EndOfStream |
                 PlayerEvent.StartOfStream |
                 PlayerEvent.StateChange |
@@ -128,6 +129,7 @@ namespace Banshee.PlaybackController
         {
             switch (args.Event) {
                 case PlayerEvent.StartOfStream:
+                    CurrentTrack = player_engine.CurrentTrack;
                     consecutive_errors = 0;
                     break;
                 case PlayerEvent.EndOfStream:
@@ -143,7 +145,11 @@ namespace Banshee.PlaybackController
 
                     CancelErrorTransition ();
                     // TODO why is this so long? any reason not to be instantaneous?
-                    Application.RunTimeout (250, EosTransition);
+                    error_transition_id = Application.RunTimeout (250, delegate {
+                        EosTransition ();
+                        RequestTrackHandler ();
+                        return true;
+                    });
                     break;
                 case PlayerEvent.StateChange:
                     if (((PlayerEventStateChangeArgs)args).Current != PlayerState.Loading) {
@@ -164,6 +170,9 @@ namespace Banshee.PlaybackController
                         transition_track_started = true;
                     }
                     break;
+                case PlayerEvent.RequestNextTrack:
+                    RequestTrackHandler ();
+                    break;
             }
         }
 
@@ -176,13 +185,19 @@ namespace Banshee.PlaybackController
         }
 
         private bool EosTransition ()
+        {
+            player_engine.IncrementLastPlayed ();
+            return true;
+        }
+
+        private bool RequestTrackHandler ()
         {
             if (!StopWhenFinished) {
                 if (RepeatMode == PlaybackRepeatMode.RepeatSingle) {
                     QueuePlayTrack ();
                 } else {
                     last_was_skipped = false;
-                    Next ();
+                    Next (RepeatMode == PlaybackRepeatMode.RepeatAll, false);
                 }
             } else {
                 OnStopped ();
@@ -211,21 +226,28 @@ namespace Banshee.PlaybackController
 
         public void Next ()
         {
-            Next (RepeatMode == PlaybackRepeatMode.RepeatAll);
+            Next (RepeatMode == PlaybackRepeatMode.RepeatAll, true);
         }
 
         public void Next (bool restart)
+        {
+            Next (restart, true);
+        }
+
+        public void Next (bool restart, bool changeImmediately)
         {
             CancelErrorTransition ();
 
             Source = NextSource;
             raise_started_after_transition = true;
 
-            player_engine.IncrementLastPlayed ();
+            if (changeImmediately) {
+                player_engine.IncrementLastPlayed ();
+            }
 
-            if (Source is IBasicPlaybackController && ((IBasicPlaybackController)Source).Next (restart)) {
+            if (Source is IBasicPlaybackController && ((IBasicPlaybackController)Source).Next (restart, changeImmediately)) {
             } else {
-                ((IBasicPlaybackController)this).Next (restart);
+                ((IBasicPlaybackController)this).Next (restart, changeImmediately);
             }
 
             OnTransition ();
@@ -275,34 +297,29 @@ namespace Banshee.PlaybackController
         bool IBasicPlaybackController.First ()
         {
             if (Source.Count > 0) {
-                player_engine.OpenPlay (Source.TrackModel[0]);
+                if (ShuffleMode == "off") {
+                    CurrentTrack = Source.TrackModel[0];
+                    player_engine.OpenPlay (CurrentTrack);
+                } else {
+                    ((IBasicPlaybackController)this).Next (false, true);
+                }
             }
             return true;
         }
 
-        bool IBasicPlaybackController.Next (bool restart)
+        bool IBasicPlaybackController.Next (bool restart, bool changeImmediately)
         {
-            TrackInfo tmp_track = CurrentTrack;
-
-            if (next_stack.Count > 0) {
-                CurrentTrack = next_stack.Pop ();
-                if (tmp_track != null) {
-                    previous_stack.Push (tmp_track);
-                }
-            } else {
-                TrackInfo next_track = QueryTrack (Direction.Next, restart);
-                if (next_track != null) {
-                    if (tmp_track != null) {
-                        previous_stack.Push (tmp_track);
-                    }
-                } else {
-                    return true;
-                }
-
-                CurrentTrack = next_track;
+            if (CurrentTrack != null) {
+                previous_stack.Push (CurrentTrack);
             }
 
-            QueuePlayTrack ();
+            CurrentTrack = CalcNextTrack (Direction.Next, restart);
+            if (!changeImmediately) {
+                // A RequestNextTrack event should always result in SetNextTrack being called.  null is acceptable.
+                player_engine.SetNextTrack (CurrentTrack);
+            } else if (CurrentTrack != null) {
+                QueuePlayTrack ();
+            }
             return true;
         }
 
@@ -312,25 +329,32 @@ namespace Banshee.PlaybackController
                 next_stack.Push (current_track);
             }
 
-            if (previous_stack.Count > 0) {
-                CurrentTrack = previous_stack.Pop ();
-            } else {
-                TrackInfo track = CurrentTrack = QueryTrack (Direction.Previous, restart);
-                if (track != null) {
-                    CurrentTrack = track;
-                } else {
-                    return true;
-                }
+            CurrentTrack = CalcNextTrack (Direction.Previous, restart);
+            if (CurrentTrack != null) {
+                QueuePlayTrack ();
             }
 
-            QueuePlayTrack ();
             return true;
         }
 
+        private TrackInfo CalcNextTrack (Direction direction, bool restart)
+        {
+            if (direction == Direction.Previous) {
+                if (previous_stack.Count > 0) {
+                    return previous_stack.Pop ();
+                }
+            } else if (direction == Direction.Next) {
+                if (next_stack.Count > 0) {
+                    return next_stack.Pop ();
+                }
+            }
+            return QueryTrack (direction, restart);
+        }
+
         private TrackInfo QueryTrack (Direction direction, bool restart)
         {
             Log.DebugFormat ("Querying model for track to play in {0}:{1} mode", ShuffleMode, direction);
-            return ShuffleMode == PlaybackShuffleMode.Linear
+            return ShuffleMode == "off"
                 ? QueryTrackLinear (direction, restart)
                 : QueryTrackRandom (ShuffleMode, restart);
         }
@@ -361,12 +385,12 @@ namespace Banshee.PlaybackController
             }
         }
 
-        private TrackInfo QueryTrackRandom (PlaybackShuffleMode mode, bool restart)
+        private TrackInfo QueryTrackRandom (string shuffle_mode, bool restart)
         {
             var track_shuffler = Source.TrackModel as Banshee.Collection.Database.DatabaseTrackListModel;
             TrackInfo track = track_shuffler == null
                 ? Source.TrackModel.GetRandom (source_set_at)
-                : track_shuffler.GetRandom (source_set_at, mode, restart, last_was_skipped, Banshee.Collection.Database.Shuffler.Playback);
+                : track_shuffler.GetRandom (source_set_at, shuffle_mode, restart, last_was_skipped, Banshee.Collection.Database.Shuffler.Playback);
             // Reset to default of true, only ever set to false by EosTransition
             last_was_skipped = true;
             return track;
@@ -475,17 +499,33 @@ namespace Banshee.PlaybackController
             }
         }
 
-        public PlaybackShuffleMode ShuffleMode {
+        public string ShuffleMode {
             get { return shuffle_mode; }
             set {
                 shuffle_mode = value;
-                EventHandler<EventArgs<PlaybackShuffleMode>> handler = ShuffleModeChanged;
+
+                // If the user changes the shuffle mode, she expects the "Next"
+                // button to behave according to the new selection. See bgo#528809
+                next_stack.Clear ();
+
+                var handler = ShuffleModeChanged;
                 if (handler != null) {
-                    handler (this, new EventArgs<PlaybackShuffleMode> (shuffle_mode));
+                    handler (this, new EventArgs<string> (shuffle_mode));
                 }
             }
         }
 
+        string prev_shuffle;
+        public void ToggleShuffle ()
+        {
+            if (ShuffleMode == "off") {
+                ShuffleMode = prev_shuffle ?? "song";
+            } else {
+                prev_shuffle = ShuffleMode;
+                ShuffleMode = "off";
+            }
+        }
+
         public PlaybackRepeatMode RepeatMode {
             get { return repeat_mode; }
             set {
@@ -497,6 +537,17 @@ namespace Banshee.PlaybackController
             }
         }
 
+        PlaybackRepeatMode? prev_repeat;
+        public void ToggleRepeat ()
+        {
+            if (RepeatMode == PlaybackRepeatMode.None) {
+                RepeatMode = prev_repeat != null ? prev_repeat.Value : PlaybackRepeatMode.RepeatAll;
+            } else {
+                prev_repeat = RepeatMode;
+                RepeatMode = PlaybackRepeatMode.None;
+            }
+        }
+
         public bool StopWhenFinished {
             get { return stop_when_finished; }
             set { stop_when_finished = value; }
