@@ -28,6 +28,7 @@
 //
 
 using System;
+using System.Linq;
 using System.Text;
 using System.Collections.Generic;
 
@@ -422,6 +423,7 @@ namespace Banshee.Sources
         public void RemoveTrack (int index)
         {
             if (index != -1) {
+                first_nonremoved_track = TrackModel[index - 1];
                 RemoveTrackRange (track_model, new RangeCollection.Range (index, index));
                 OnTracksRemoved ();
             }
@@ -432,58 +434,80 @@ namespace Banshee.Sources
             RemoveTrack (track_model.IndexOf (track));
         }
 
-        public virtual void RemoveSelectedTracks ()
+        public void RemoveTracks (Selection selection)
         {
-            RemoveSelectedTracks (track_model);
+            RemoveTracks (track_model, selection);
         }
 
-        public virtual void RemoveSelectedTracks (DatabaseTrackListModel model)
+        public void RemoveTracks (DatabaseTrackListModel model, Selection selection)
         {
-            WithTrackSelection (model, RemoveTrackRange);
+            FindFirstNotRemovedTrack (model, selection);
+            WithTrackSelection (model, selection, RemoveTrackRange);
             OnTracksRemoved ();
         }
 
-        public virtual void DeleteSelectedTracks ()
+        protected void FindFirstNotRemovedTrack (DatabaseTrackListModel model, Selection selection)
         {
-            DeleteSelectedTracks (track_model);
+            first_nonremoved_track = null;
+
+            var playback_src = ServiceManager.PlaybackController.Source as DatabaseSource;
+            if (playback_src != this && playback_src.Parent != this)
+                return;
+
+            int i = model.IndexOf (ServiceManager.PlayerEngine.CurrentTrack);
+            if (!selection.Contains (i))
+                return;
+
+            var range = selection.Ranges.First (r => r.Start <= i && i <= r.End);
+            first_nonremoved_track = model[range.Start - 1];
         }
 
-        protected virtual void DeleteSelectedTracks (DatabaseTrackListModel model)
+        public void DeleteTracks (Selection selection)
+        {
+            DeleteTracks (track_model, selection);
+        }
+
+        public virtual void DeleteTracks (DatabaseTrackListModel model, Selection selection)
         {
             if (model == null)
                 return;
 
-            WithTrackSelection (model, DeleteTrackRange);
+            FindFirstNotRemovedTrack (model, selection);
+            WithTrackSelection (model, selection, DeleteTrackRange);
             OnTracksDeleted ();
         }
 
-        public virtual bool AddSelectedTracks (Source source)
+        public bool AddSelectedTracks (Source source)
         {
-            if (!AcceptsInputFromSource (source))
+            var track_src = source as ITrackModelSource;
+            return AddSelectedTracks (source, track_src == null ? null : track_src.TrackModel.Selection);
+        }
+
+        public virtual bool AddSelectedTracks (Source source, Selection selection)
+        {
+            if (!AcceptsInputFromSource (source) || selection.Count == 0)
                 return false;
 
             DatabaseTrackListModel model = (source as ITrackModelSource).TrackModel as DatabaseTrackListModel;
             if (model == null) {
                 return false;
             }
-            WithTrackSelection (model, AddTrackRange);
+
+            WithTrackSelection (model, selection, AddTrackRange);
             OnTracksAdded ();
             OnUserNotifyUpdated ();
             return true;
         }
 
-        public virtual bool AddAllTracks (Source source)
+        public bool AddAllTracks (Source source)
         {
-            if (!AcceptsInputFromSource (source) || source.Count == 0)
-                return false;
-
-            DatabaseTrackListModel model = (source as ITrackModelSource).TrackModel as DatabaseTrackListModel;
-            lock (model) {
-                AddTrackRange (model, new RangeCollection.Range (0, source.Count));
+            var track_src = source as ITrackModelSource;
+            if (track_src != null) {
+                var selection = new Selection () { MaxIndex = track_src.TrackModel.Selection.MaxIndex };
+                selection.SelectAll ();
+                return AddSelectedTracks (source, selection);
             }
-            OnTracksAdded ();
-            OnUserNotifyUpdated ();
-            return true;
+            return false;
         }
 
         public virtual void RateSelectedTracks (int rating)
@@ -547,6 +571,7 @@ namespace Banshee.Sources
             }
         }
 
+        TrackInfo first_nonremoved_track = null;
         protected virtual void OnTracksDeleted ()
         {
             PruneArtistsAlbums ();
@@ -554,12 +579,37 @@ namespace Banshee.Sources
             foreach (PrimarySource psource in PrimarySources) {
                 psource.NotifyTracksDeleted ();
             }
+            SkipTrackIfRemoved ();
         }
 
         protected virtual void OnTracksRemoved ()
         {
             PruneArtistsAlbums ();
             Reload ();
+            SkipTrackIfRemoved ();
+        }
+
+        protected void SkipTrackIfRemoved ()
+        {
+            var playback_src = ServiceManager.PlaybackController.Source as DatabaseSource;
+            if (playback_src == null || (playback_src != this && playback_src.Parent != this))
+                return;
+
+            var track = ServiceManager.PlayerEngine.CurrentTrack as DatabaseTrackInfo;
+            if (track == null || playback_src.DatabaseTrackModel.Contains (track))
+                return;
+
+            Log.DebugFormat ("Playing track was removed from {0}, so stopping it", this);
+            if (ServiceManager.PlaybackController.StopWhenFinished) {
+                ServiceManager.PlayerEngine.Close ();
+            } else {
+                // Set the PriorTrack to the track before the one we removed, so that, if we're in non-shuffle mode,
+                // we'll pick the first non-removed track after the one that was playing
+                if (first_nonremoved_track != null) {
+                    ServiceManager.PlaybackController.PriorTrack = first_nonremoved_track;
+                }
+                ServiceManager.PlaybackController.Next ();
+            }
         }
 
         // If we are a PrimarySource, return ourself and our children, otherwise if our Parent
@@ -649,7 +699,11 @@ namespace Banshee.Sources
 
         protected void WithTrackSelection (DatabaseTrackListModel model, TrackRangeHandler handler)
         {
-            Selection selection = model.Selection;
+            WithTrackSelection (model, model.Selection, handler);
+        }
+
+        protected void WithTrackSelection (DatabaseTrackListModel model, Selection selection, TrackRangeHandler handler)
+        {
             if (selection.Count == 0)
                 return;
 
