@@ -190,9 +190,10 @@ namespace Banshee.Sources
             get { return SourceMergeType.None; }
         }
 
-        public virtual void SetParentSource (Source parent)
+        public virtual bool SetParentSource (Source parent)
         {
             this.parent = parent;
+            return true;
         }
 
         public virtual bool ContainsChildSource (Source child)
@@ -206,9 +207,10 @@ namespace Banshee.Sources
         {
             lock (Children) {
                 if (!child_sources.Contains (child)) {
-                    child.SetParentSource (this);
-                    child_sources.Add (child);
-                    OnChildSourceAdded (child);
+                    if (child.SetParentSource (this)) {
+                        child_sources.Add (child);
+                        OnChildSourceAdded (child);
+                    }
                 }
             }
         }
@@ -221,13 +223,6 @@ namespace Banshee.Sources
                 }
 
                 child_sources.Remove (child);
-
-                if (ServiceManager.SourceManager.ActiveSource == child) {
-                    if (CanActivate) {
-                        ServiceManager.SourceManager.SetActiveSource (this);
-                    }
-                }
-
                 OnChildSourceRemoved (child);
             }
         }
@@ -271,7 +266,11 @@ namespace Banshee.Sources
 
                     int i = 0;
                     foreach (Source child in child_sources) {
-                        child.Order = i++;
+                        // Leave children with negative orders alone, so they can be manually
+                        // placed at the top
+                        if (child.Order >= 0) {
+                            child.Order = i++;
+                        }
                     }
                 }
             }
@@ -357,8 +356,7 @@ namespace Banshee.Sources
             }
         }
 
-
-        protected virtual void PushMessage (SourceMessage message)
+        public void PushMessage (SourceMessage message)
         {
             lock (this) {
                 messages.Insert (0, message);
@@ -368,7 +366,7 @@ namespace Banshee.Sources
             OnMessageNotify ();
         }
 
-        protected virtual SourceMessage PopMessage ()
+        protected SourceMessage PopMessage ()
         {
             try {
                 lock (this) {
@@ -386,7 +384,7 @@ namespace Banshee.Sources
             }
         }
 
-        protected virtual void ClearMessages ()
+        protected void ClearMessages ()
         {
             lock (this) {
                 if (messages.Count > 0) {
@@ -439,7 +437,7 @@ namespace Banshee.Sources
             });
         }
 
-        protected virtual void RemoveMessage (SourceMessage message)
+        public void RemoveMessage (SourceMessage message)
         {
             lock (this) {
                 if (messages.Remove (message)) {
@@ -451,6 +449,10 @@ namespace Banshee.Sources
 
         private void HandleMessageUpdated (object o, EventArgs args)
         {
+            if (CurrentMessage == o && CurrentMessage.IsHidden) {
+                PopMessage ();
+            }
+
             OnMessageNotify ();
         }
 
@@ -550,7 +552,7 @@ namespace Banshee.Sources
         public string UniqueId {
             get {
                 if (unique_id == null && type_unique_id == null) {
-                    Log.ErrorFormat ("Creating Source.UniqueId for {0}, but TypeUniqueId is null; trace is {1}", this.Name, System.Environment.StackTrace);
+                    Log.ErrorFormat ("Creating Source.UniqueId for {0} (type {1}), but TypeUniqueId is null; trace is {2}", this.Name, GetType ().Name, System.Environment.StackTrace);
                 }
                 return unique_id ?? (unique_id = String.Format ("{0}-{1}", this.GetType ().Name, TypeUniqueId));
             }
@@ -762,7 +764,8 @@ namespace Banshee.Sources
                 return String.Empty;
             }
 
-            builder.AppendFormat (Catalog.GetPluralString ("{0} item", "{0} items", count), count);
+            var count_str = String.Format ("{0:N0}", count);
+            builder.AppendFormat (GetPluralItemCountString (count), count_str);
 
             if (this is IDurationAggregator && StatusFormatsCount > 0) {
                 var duration = ((IDurationAggregator)this).Duration;
@@ -783,6 +786,11 @@ namespace Banshee.Sources
             return builder.ToString ();
         }
 
+        public virtual string GetPluralItemCountString (int count)
+        {
+            return Catalog.GetPluralString ("{0} item", "{0} items", count);
+        }
+
 #endregion
 
         public override string ToString ()
